import java.sql.*;
import javax.sql.*;
import java.io.*;
import java.net.*;
import java.util.Vector;
import java.util.HashMap;
import java.math.BigDecimal;

public class DBConnect implements DatabaseConnect
{
 private Connection connect;
 private boolean initConnection = false;
 private final String defdriver = "com.mysql.jdbc.Driver";
 private final String defuser   = "edisonch";
 private final String defurl    = "jdbc:mysql://alien:3306/";
 private final String defpwd    = "gblj21mysql03";
 private StringBuffer driver = new StringBuffer();
 private StringBuffer user = new StringBuffer();
 private StringBuffer url = new StringBuffer();
 private StringBuffer pwd = new StringBuffer();

 public DBConnect()
 {
  driver.append(defdriver);
  user.append(defuser);
  url.append(defurl);
  pwd.append(defpwd);
 }

 public DBConnect(String drv,String url2,String usr,String pass)
 {
  resetInfo(drv,url2,usr,pass);
 }

 public void close()
 {
  if(!initConnection)
   return;
  try
  {
   driver.delete(0,driver.length());
   user.delete(0,user.length());
   url.delete(0,url.length());
   pwd.delete(0,pwd.length());
   connect.close();
  }
  catch(Exception ex)
  {
   System.out.println("DBConnect failed to close : "+ex.getMessage());
  }
 }

 public void resetInfo(String drv,String url2,String usr,String pass)
 {
  if(drv!=null)
   driver.append(drv);
  if(url2!=null)
   url.append(url2);
  if(usr!=null)
   user.append(usr);
  if(pass!=null)
   pwd.append(pass);
 }

 public Vector countNames(Encap en)
 {
  int size = en.getSize();
  Vector names = new Vector();
  for(int i=0;i<size;i++)
   if(!names.contains(en.getName(i)))
    names.add(en.getName(i));
  return names;
 }

 public boolean makeConnection(String dbname)
 {
  try
  {
   Class.forName(driver.toString()).newInstance();
   url.append(dbname);
   connect = DriverManager.getConnection(url.toString(),
		 user.toString(),pwd.toString());
  }
  catch(SQLException ex)
  {
   System.out.println("Error Connecting to database : "+ex.getMessage());
   initConnection = false;
   return false;
  }
  catch(ClassNotFoundException ce)
  {
   System.out.println("JDBC driver class not found : "+ce.getMessage());
   initConnection = false;
   return false;
  }
  catch(Exception e)
  {
   System.out.println("Exception : "+e.getMessage());
   initConnection = false;
   return false;
  }
  initConnection = true;
  return true;
 }

/**
 * InsertMulti's method is to insert multiple encap's data into database.
 * It supports transaction
 */
 public void InsertMulti(Vector v) throws SQLException
 {
  if(!initConnection)
   throw new SQLException("No initial connection to database establish");
	if(v == null)
	 throw new SQLException("Received null Vector");
  int size = v.size();
  Statement stmt = null;
  try
  {
   connect.setAutoCommit(false);
   stmt = connect.createStatement();
   for(int i=0;i<size;i++)
    workInsert(stmt,(Encap)v.elementAt(i));
   connect.commit();
  }
  catch(SQLException sqle)
  {
   connect.rollback();
   throw sqle;
  }
  finally
  {
   stmt.close();
  }
  return;
 } // End of InsertMulti

/**
 * InsertData's method is to insert data one by one (Encap by Encap) into the
 * database. The order of the column names and values is important into
 * filling the database from Encap. Also Encap packages have to follow their
 * biggest package in the their own container (Vector). If the biggest is
 * 3 tuples, then every package has to be 3 tuples long.
 * @param Encap en
 * @return none
 */
 public void InsertData(Encap en) throws SQLException
 {
  if(!initConnection)
   throw new SQLException("No initial connection to database establish");
  Statement stmt = null;
  try
  {
   connect.setAutoCommit(false);
   stmt = connect.createStatement();
   workInsert(stmt,en);
   connect.commit();
  }
  catch(SQLException sqle)
  {
   connect.rollback();
   throw sqle;
  }
  finally
  {
   stmt.close();
  }
  return;
 }

/**
 * workInsert's method is to string that makes up the INSERT syntax
 * given the Encap and the Vectors of names (for the column name field)
 * @param Encap en - contains the needed table name
 *        Vector names - contains all the column names field of Encap
 * @return INSERT syntax of the given Encap
 */
 private void workInsert(Statement stmt,Encap en) throws SQLException
 {
  if(en == null)
   throw new SQLException("Received null Encapsulation");
  if(en.getTable() == null)
   throw new SQLException("Received null tableName");
  int size = en.getSize();
  StringBuffer str = new StringBuffer();
  StringBuffer str2= new StringBuffer();
  str2.append("INSERT INTO "+en.getTable()+" (");
  for(int i=0;i<size;i++)
  {
   str2.append(en.getName(i));
   switch(en.getType(i))
   {
    case Encap.INT : str.append(en.getIntValue(i)); break;
    case Encap.STR : str.append("\""+en.getStringValue(i)+"\""); break;
    case Encap.FLT : str.append(en.getFloatValue(i)); break;
    case Encap.DBL : str.append(en.getDoubleValue(i)); break;
    case Encap.TIM :
    case Encap.DAT :
    case Encap.VAR : str.append(en.getStringValue(i)); break;
    case Encap.LNG : str.append(en.getLongValue(i)); break;
    case Encap.DEC : str.append(en.getValue(i)); break;
   }
   if(i!=size-1)
   {
    str.append(",");
    str2.append(",");
   }
   else
    str2.append(") ");
  }
  str2.append(" VALUES ("+str.toString()+"); ");
  stmt.executeUpdate(str2.toString());
  str.delete(0,str.length());
  if(en.getVariable()!=null)
  {
   str.append(en.getVariable());
   stmt.executeUpdate(str.toString());
  }
  str.delete(0,str.length());
  return;
 }

/**
 * work
 */
 private void work(Statement stmt,Encap en) throws SQLException
 {
  int size = en.getSize();
  StringBuffer str = new StringBuffer("UPDATE "+en.getTable()+" SET ");
  for(int i=0;i<size;i++)
  {
   switch(en.getType(i))
   {
    case Encap.INT: str.append(en.getName(i)+"="+en.getIntValue(i)); break;
    case Encap.STR: str.append(en.getName(i)+"=\""+en.getStringValue(i)+"\"");
        	    break;
    case Encap.FLT: str.append(en.getName(i)+"="+en.getFloatValue(i)); break;
    case Encap.DBL: str.append(en.getName(i)+"="+en.getDoubleValue(i)); break;
    case Encap.TIM:
    case Encap.DAT: 
    case Encap.VAR: str.append(en.getName(i)+"="+en.getStringValue(i)); break;
    case Encap.LNG: str.append(en.getName(i)+"="+en.getLongValue(i)); break;
    case Encap.DEC: str.append(en.getName(i)+"="+en.getValue(i)); break;
   }
   if(i!=size-1) str.append(",");
  }
  if(en.getCondition()!=null)
   str.append(" "+en.getCondition());
  else
   str.append(";");
	System.out.println("work : "+str.toString());
  stmt.executeUpdate(str.toString());
 }


 public void UpdateMulti(Vector v) throws SQLException
 {
  if(!initConnection)
    throw new SQLException("No initial connection to database establish");
  if(v == null) throw new SQLException("Passed in value is null");
  int size = v.size();
  Encap en = null;
  Statement stmt = null;
  try
  {
   connect.setAutoCommit(false);
   stmt = connect.createStatement();
   for(int i=0;i<size;i++)
   {
    en = (Encap)v.elementAt(i);
    work(stmt,en);
    en.clear();
   }
   connect.commit();
  }
  catch(SQLException sqle)
  {
   connect.rollback();
   throw sqle;
  }
  finally
  {
   stmt.close();
  }
 }

 public void UpdateData(Encap en) throws SQLException
 {
  if(!initConnection)
    throw new SQLException("No initial connection to database establish");
  if(en == null) throw new SQLException("Passed in value is null");
  Statement stmt = null; 
  try
  {
   connect.setAutoCommit(false);
   stmt = connect.createStatement();
   work(stmt,en);
   connect.commit();
  }
  catch(SQLException sqle)
  {
   connect.rollback();
   throw sqle;
  }
  finally
  {
   stmt.close();
  }
 }


/**
 * QueryMulti
 */
 public Vector QueryMulti(Encap en) throws SQLException
 {
  if(!initConnection)
    throw new SQLException("No initial connection to database establish");
  if(en == null) throw new SQLException("Passed in value is null");
  Vector names = countNames(en);
  ResultSet result = null;
  Statement stmt = null;
  Vector v = null;
  try
  {
   stmt = connect.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,
                                  ResultSet.CONCUR_UPDATABLE);
   result = stmt.executeQuery(formSelect(names,en));
   if(result.next())
     v = workQuery(result);
  }
  catch(SQLException sqle)
  {
   System.out.println("SQLException at QueryMulti: "+sqle.getMessage());
   throw sqle;
  }
  finally
  {
   result.close();
   stmt.close();
  }
  return v;
 }

 private Vector workQuery(ResultSet result) throws SQLException
 {
  Encap en = new Encap();
  Vector v = new Vector();
  ResultSetMetaData rsmd = result.getMetaData();
  int columnSize = rsmd.getColumnCount();
  do
  {
   en.setTable(rsmd.getTableName(1));
   for(int i=0;i<columnSize;i++)
   {
    switch(rsmd.getColumnType(i+1))
    {
     case java.sql.Types.CHAR    :
     case java.sql.Types.VARCHAR :
      en.setNameVal(rsmd.getColumnName(i+1),result.getString(i+1));
      break;
     case java.sql.Types.SMALLINT:
     case java.sql.Types.TINYINT :
     case java.sql.Types.INTEGER : 
      en.setNameVal(rsmd.getColumnName(i+1),result.getInt(i+1));
      break;
     case java.sql.Types.FLOAT   :
      en.setNameVal(rsmd.getColumnName(i+1),result.getFloat(i+1));
      break;
     case java.sql.Types.BIGINT  :
      en.setNameVal(rsmd.getColumnName(i+1),result.getLong(i+1));
      break;
     case java.sql.Types.DECIMAL :
      en.setNameVal(rsmd.getColumnName(i+1),result.getBigDecimal(i+1));
      break;
     case java.sql.Types.DOUBLE  :
      en.setNameVal(rsmd.getColumnName(i+1),result.getDouble(i+1));
      break;
     case java.sql.Types.DATE    :
      en.setName(rsmd.getColumnName(i+1));
      if(result.getDate(i+1) == null)
       en.setValue("0000-00-00");
      else 
       en.setValue(result.getDate(i+1).toString());
      break;
     case java.sql.Types.TIME    :
      en.setName(rsmd.getColumnName(i+1));
      if(result.getTime(i+1) == null)
       en.setValue("00:00:00");
      else
       en.setValue(result.getTime(i+1).toString());
     break;
     default : 
      System.out.println("default : "+rsmd.getColumnType(i+1));
      throw new SQLException("Default error in switch statement");
    } // end of switch
   } // end of for-loop
   v.add(en.clone());
   en.clear();
  } while(result.next()); // end of while-loop
  return v;
 }

 private String formSelect(Vector names,Encap en) throws SQLException
 {
  if((names == null)||(names.size()==0))
   return null;
  StringBuffer str = new StringBuffer();
  str.append("SELECT ");
  int psize = names.size();
  if(psize>1)
  {
   for(int i=0;i<psize;i++)
   {
    str.append((String)(names.elementAt(i)));
    if(i<psize-1)
     str.append(",");
    else
     str.append(" ");
   }
  }
  else
   str.append((String)names.firstElement()+" ");
  str.append("FROM "+en.getTable());
  if(en.getCondition()!=null)
    str.append(" "+en.getCondition());
  return str.toString();
 }

 public Encap QuerySingle(Encap en) throws SQLException
 {
  if(!initConnection)
    throw new SQLException("No initial connection to database establish");
  if(en == null) throw new SQLException("Passed in value is null");
  Vector names = countNames(en);
  Encap en2 = null;
  ResultSet result = null;
  Statement stmt = null;
  ResultSetMetaData rsmd = null;
  try
  {
   connect.setTransactionIsolation(Connection.TRANSACTION_SERIALIZABLE);
   stmt = connect.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,
     				  ResultSet.CONCUR_UPDATABLE);
   result = stmt.executeQuery(formSelect(names,en));
   if(result.next())
   {
    rsmd = result.getMetaData();
    int columnSize = rsmd.getColumnCount();
    en2 = new Encap();
    do
    {
     en2.setTable(rsmd.getTableName(1));
     for(int i=0;i<columnSize;i++)
     {
      switch(rsmd.getColumnType(i+1))
      {
       case java.sql.Types.CHAR    :
       case java.sql.Types.VARCHAR :
        en2.setNameVal(rsmd.getColumnName(i+1),result.getString(i+1));
        break;
       case java.sql.Types.SMALLINT:
       case java.sql.Types.TINYINT :
       case java.sql.Types.INTEGER :
        en2.setNameVal(rsmd.getColumnName(i+1),result.getInt(i+1));
        break;
       case java.sql.Types.BIGINT  :
	en2.setNameVal(rsmd.getColumnName(i+1),result.getLong(i+1));
	break;
       case java.sql.Types.FLOAT   :
        en2.setNameVal(rsmd.getColumnName(i+1),result.getFloat(i+1));
        break;
       case java.sql.Types.DECIMAL :
	en2.setNameVal(rsmd.getColumnName(i+1),result.getBigDecimal(i+1));
	break;
       case java.sql.Types.DOUBLE  :
        en2.setNameVal(rsmd.getColumnName(i+1),result.getDouble(i+1));
        break;
       case java.sql.Types.TIME    :
	if(result.getTime(i+1) == null)
         en2.setNameVal(rsmd.getColumnName(i+1),"00:00:00");
	else 
         en2.setNameVal(rsmd.getColumnName(i+1),
	 result.getTime(i+1).toString());
        break;
       case java.sql.Types.DATE    :
	if(result.getDate(i+1) == null)
         en2.setNameVal(rsmd.getColumnName(i+1),"0000-00-00");
	else 
         en2.setNameVal(rsmd.getColumnName(i+1),
	 result.getDate(i+1).toString());
        break;
       default : 
      	throw new SQLException("Default error in switch statement");
      } // end of switch
     } // end of for-loop
    } while(result.next());
   } // end of if
  }
  catch(SQLException sqle)
  {
   System.out.println("SQLException at QuerySingle: "+sqle.getMessage());
   throw sqle;
  }
  finally
  {
   if(result!=null) result.close();
   if(stmt!=null) stmt.close();
  }
  return en2;
 }

 public Vector QueryMulti(String table,String col,String condition)
  throws SQLException
 {
  if(!initConnection)
    throw new SQLException("No initial connection to database establish");
  Encap en = new Encap(table);
  int p = col.indexOf(',');
  if(p<0) en.setName(col);
  else
   en = phraseNames(p,col,en);
  if(condition!=null) en.setCondition(condition);
  return QueryMulti(en);
 }

 public Encap QuerySingle(String table,String col,String condition)
  throws SQLException
 {
  if(!initConnection)
    throw new SQLException("No initial connection to database establish");
  Encap en = new Encap(table);
  int p = col.indexOf(',');
  if(p<0) en.setName(col);
  else
   en = phraseNames(p,col,en);
  if(condition!=null) en.setCondition(condition);
  return QuerySingle(en);
 }

 public void DeleteSingle(String table,String condition) throws SQLException
 {
  if(!initConnection)
   throw new SQLException("No initial connection to database establish");
  Encap en = new Encap(table);
  en.setCondition(condition);
  DeleteSingle((Encap)en.clone());
  en.clear();
 }

 public void DeleteSingle(Encap en) throws SQLException
 {
  if(!initConnection)
   throw new SQLException("No initial connection to database establish");
  if(en == null) throw new SQLException("Passed null value");
  StringBuffer q = new StringBuffer(32);
  q.append("DELETE FROM "+en.getTable()+" "+en.getCondition());
  Statement stmt = null;
  try
  {
   connect.setAutoCommit(false);
   stmt = connect.createStatement();
   stmt.executeUpdate(q.toString());
	 connect.commit();
  }
  catch(SQLException sqle)
  {
   connect.rollback();
   throw sqle;
  }
  finally
  {
   stmt.close();
  }
 }

 public void UpdateSpecial(String table,String setting,String condition)
	throws SQLException
 {
  if(!initConnection)
   throw new SQLException("No initial connection to database establish");
	if(table == null || setting == null || condition == null)
	 throw new SQLException("One of the parameters is null");
	StringBuffer q = new StringBuffer(64);
	q.append("UPDATE "+table+" SET "+setting+" "+condition);
  Statement stmt = null;
	try
	{
   connect.setAutoCommit(false);
   stmt = connect.createStatement();
   stmt.executeUpdate(q.toString());
	 connect.commit();
	}
	catch(SQLException sqle)
	{
   connect.rollback();
   throw sqle;
	}
	finally
	{
   stmt.close();
	}
 }

 public void UpdateSpecialMany(Vector v) throws SQLException
 {
  if(!initConnection)
   throw new SQLException("No initial connection to database establish");
	if(v == null)
	 throw new SQLException("null Vector");
  if(v.isEmpty())
	 throw new SQLException("Empty Vector");
  Statement stmt = null;
	try
	{
   connect.setAutoCommit(false);
   stmt = connect.createStatement();
	 while(!v.isEmpty()) 
    stmt.executeUpdate((String)v.remove(0));
	 connect.commit();
	}
	catch(SQLException sqle)
	{
   connect.rollback();
   throw sqle;
	}
	finally
	{
   stmt.close();
	}
 }

 private Encap phraseNames(int p,String col,Encap en)
 {
  int start = 0;
  do
  {
   en.setName(col.substring(start,p));
   start = p;
   p = col.indexOf(',',start+1);
   if(p<0) en.setName(col.substring(start+1,col.length()));
  } while(p>0);
  return en;
 }

 public static void viewEncap(Encap en)
 {
  int size = en.getSize();
  System.out.println("Encap Table: "+en.getTable());
  for(int i=0;i<size;i++)
  {
   System.out.print(i+" EncapName: "+en.getName(i));
   if(en.getValueSize()>0)
    System.out.print(" EncapValue: "+en.getValue(i));
   System.out.println("");
  }
  if(en.getVariable()!=null)
   System.out.print("variable: "+en.getVariable()+" ");
  if(en.getCondition()!=null)
   System.out.print("condition: "+en.getCondition()+" ");
	System.out.println("");
 }

}
