import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.plaf.basic.BasicArrowButton;
import java.util.Hashtable;

/**
 * SortedColumnHeaderRenderer's class is copied from "Professional JAVA 
 * Programming" by Brett Spell.
 */

public class SortedColumnHeaderRenderer implements TableCellRenderer
{
 protected TableCellRenderer tabRenderer;
 protected SortedTableModel sortedModel;

 public static final int NONE = 0;
 public static final int DOWN = 1;
 public static final int UP   = 2;
 public int pushedColumn;
 public Hashtable state;
 
 public SortedColumnHeaderRenderer(SortedTableModel model,
   TableCellRenderer renderer)
 {
  pushedColumn = -1;
  state = new Hashtable();
  sortedModel = model;
  tabRenderer = renderer;
 }

 public SortedColumnHeaderRenderer(SortedTableModel model)
 { 
  this(model,null);
 }

 public Component getTableCellRendererComponent(JTable table,
   Object value,boolean isSelected,boolean hasFocus,int row,
   int column)
 {
  Component text;
  JPanel panel = new JPanel();
  panel.setLayout(new BorderLayout());
  if(tabRenderer != null)
   text = tabRenderer.getTableCellRendererComponent(table,
     value,isSelected,hasFocus,row,column);
  else
  {
   text = new JLabel((String)value,JLabel.CENTER);
   LookAndFeel.installColorsAndFont((JComponent)text,
     "TableHeader.background","TableHeader.foreground","TableHeader.font");
  }
  panel.add(text,BorderLayout.CENTER);
  int position = NONE;
  if(state.get(new Integer(column))!=null)
   position = ((Integer)state.get(new Integer(column))).intValue();
  if(position == UP)
  {
   BasicArrowButton bab = new BasicArrowButton(SwingConstants.NORTH);  
   panel.add(bab,BorderLayout.WEST);
  }
  else if(position == DOWN)
  {
   BasicArrowButton bab = new BasicArrowButton(SwingConstants.SOUTH);
   panel.add(bab,BorderLayout.WEST);
  }
  LookAndFeel.installBorder(panel,"TableHeader.cellBorder");
  return panel;
 }

 public void setPressedColumn(int col)
 {
  pushedColumn = col;
 }

 public void setSelectedColumn(int col)
 {
  if(col < 0) return;
  Integer value = null;
  Object obj = state.get(new Integer(col));
  if(obj == null)
   value = new Integer(DOWN);
  else
  {
   if(((Integer)obj).intValue() == DOWN)
    value = new Integer(UP);
   else
    value = new Integer(DOWN);
  }
  state.clear();
  state.put(new Integer(col),value);
 }

 public int getState(int col)
 {
  int retValue;
  Object obj = state.get(new Integer(col));
  if(obj == null)
   retValue = NONE;
  else
  {
   if(((Integer)obj).intValue() == DOWN)
    retValue = DOWN;
   else
    retValue = UP;
  }
  return retValue;
 }
}
