import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.util.Vector;
import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.HashMap;

public class ViewByTransKey
{
 private JTable table;
 private JFrame frame;
 private tableData td;
 private JButton refreshButton;
 private JButton closeButton;

 private int key = 0;
 private JList head = null;
 private Vector data = null;
 private String condition = null;
 private StringBuffer error = new StringBuffer();
 private SortedColumnHeaderRenderer schr = null;
 private SortedTableModel sortedModel = null;
 
 public ViewByTransKey(int key)
 {
  if(key == 0 || key < 0)
  {
   Mandalatama.errorDisplay("Received invalid key: "+key,
   "ERROR Receiving");
   System.gc();
   return;
  }
  this.key = key;
  getData(key);
  if(error.length()>0)
  {
   Mandalatama.infoDisplay(error.toString(),"Info");
   error.delete(0,error.length());
   System.gc();
   return;
  }
  frame = new JFrame("View by transaction key: "+key);
  frame.addWindowListener(new WindowAdapter()
  {
   public void windowClosing(WindowEvent e)
   {
    Mandalatama.infoDisplay("Click on Close in the panel instead",
     "Information");
    frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    return;
   }
  });
  frame.getContentPane().setLayout(new BorderLayout());
  frame.getContentPane().add(createMain(),BorderLayout.NORTH);
  frame.getContentPane().add(createBottom(),BorderLayout.SOUTH);
  frame.pack();
  frame.setVisible(true);
 }

 private JPanel createMain()
 {
  JPanel panel = new JPanel();
  td = new tableData(data);
  table = new JTable();
  table.setAutoCreateColumnsFromModel(false);
  table.setRowHeight(20);
  table.setModel(td);
  table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
  MultiLineHeaderRender mlhr = new MultiLineHeaderRender();
  int len = td.getColumnCount();
  for(int i=0;i<len;i++)
  {
   TableCellRenderer render = new DefaultTableCellRenderer();
   TableCellEditor editor = new DefaultCellEditor(new JTextField());
   TableColumn column;
   if(i==td.TID)
    column = new TableColumn(i,50,render,editor);
   else
    column = new TableColumn(i,50,render,editor);
   column.setHeaderRenderer(schr);
   table.addColumn(column);
  }
  panel.setLayout(new BorderLayout());
  head = new JList(new ListMod(data.size()));
  head.setFixedCellWidth(20);
  head.setFixedCellHeight(table.getRowHeight());
  head.setCellRenderer(new RowHeaderRenderer(table));
  JScrollPane jsp = new JScrollPane(table);
  jsp.setRowHeaderView(head);
  panel.add(jsp,BorderLayout.CENTER);
  panel.setPreferredSize(new Dimension(640,200));
  return panel;
 }

 private JPanel createBottom()
 {
  JPanel panel = new JPanel();
  refreshButton = new JButton("Refresh");
  closeButton = new JButton("Close");
  ButtonListener bl = new ButtonListener();
  refreshButton.addActionListener(bl);
  closeButton.addActionListener(bl);
  GridBagLayout gbl = new GridBagLayout();
  GridBagConstraints gbc = new GridBagConstraints();
  gbc.anchor = GridBagConstraints.CENTER;
  gbc.fill = GridBagConstraints.BOTH;
  gbc.gridx = GridBagConstraints.RELATIVE;
  gbc.insets = new Insets(2,2,2,2);
  panel.setLayout(gbl);
  gbc.gridwidth=2;
  gbc.gridy = 0;
  panel.add(refreshButton,gbc);
  panel.add(closeButton,gbc);
  panel.setPreferredSize(new Dimension(640,100));
  return panel;
 }

 private void getData(int key)
 {
  if(key<=0)
  {
   error.append("Received invalid key at getData\n");
   return;
  }
  DBConnect db = new DBConnect();
  try
  {
   if(!db.makeConnection("MANDALATAMA"))
    throw new SQLException("Failure to make connection to database\n");
   data = db.QueryMulti("transaction","*","WHERE transKey="+key+";");
   if(data == null)
    error.append("No transaction available for the transaction key: "+key+"\n"); 
   else
    refineResults(db);  
  }
  catch(SQLException sqle)
  {
   error.append(sqle.getMessage());
   sqle.printStackTrace(System.out);
   if(!data.isEmpty()) data.removeAllElements();
  }
  finally
  {
   db.close();
  }
 }

 private void refineResults(DBConnect db) throws SQLException
 {
  if(data == null || data.size()==0)
  {
   error.append("Data is null or empty");
   return;
  }
  int key = 0; //key
  int pkey= 0; //payment key
  int pos = 0;
  Encap e = null;
  Encap e2= null;
  Encap e3= null;
  Vector v = new Vector(data.size());
  Vector v2 = null;

  while(!data.isEmpty())
  {
   e = (Encap)data.remove(0);
   key = e.extractInteger("transKey",0);
   v.add(new transData("transaction key",String.valueOf(key)));
   v.add(new transData("transaction valid",e.extractAll("transValid")));
   pos = e.getPosition("transInvoiceNo");
   if(pos>=0)
    v.add(new transData("Invoice No",e.getValue(pos)));
   v.add(new transData("transaction description",e.extractAll("transDescription")));
   v.add(new transData("transaction date/time",
    VariousCheck.formNDate(e.extractAll("transDate"))+" "+
    e.extractAll("transTime")));
   v2 = db.QueryMulti("transPayment","*","WHERE transPaymentOfKey="+key+";");
   if(v2 != null)
   {
    while(!v2.isEmpty())
    {
     e2 = (Encap)v2.remove(0);
     pkey = e2.extractInteger("transPaymentKey",0);
     v.add(new transData("payment key",String.valueOf(pkey)));
     v.add(new transData("payment type",e2.extractAll("transPaymentType")));
     if(!VariousCheck.checkException(e2.extractAll("transPaymentMethod")))
     {
      e3 = db.QuerySingle("transDocument","*","WHERE transDocOfKey="+pkey+";");
      if(e3 != null)
      {
       v.add(new transData("payment method/doc no.",
	e2.extractAll("transPaymentMethod")+" "+
        e3.extractAll("transDocNo")));
       e3.clear();
      }
      else 
       v.add(new transData("payment method",e2.extractAll("transPaymentMethod")));
     } // end of non-cash transaction
     else // it is cash transaction
      v.add(new transData("payment method",e2.extractAll("transPaymentMethod")));
     e3 = db.QuerySingle("transRemarks","*","WHERE transRemarksOfKey="+pkey+";");
     if(e3 != null)
     {
      v.add(new transData("transaction remarks",
      e3.extractAll("transRemarksContent")));
      e3.clear();
     }

     v.add(new transData("payment curr/amount",e2.extractAll("transPaymentCurr")+
     " "+VariousCheck.formNumDelimiter(e2.extractAll("transPaymentAmnt"))));
     v.add(new transData("payment date/time",
      VariousCheck.formNDate(e2.extractAll("transPaymentDate"))+" "+
      e2.extractAll("transPaymentTime")));
     if(!e2.extractAll("transPaymentChangedDate").equalsIgnoreCase("0000-00-00"))
     {
      v.add(new transData("payment changed by id/date/time",
       e2.extractAll("transPaymentChangedBy")+" "+
       VariousCheck.formNDate(e2.extractAll("transPaymentChangedDate"))+" "+
       e2.extractAll("transPaymentChangedTime")));
     }
     e2.clear();
    }
   } // end of there is payment made
   e.clear();
  }
  data = (Vector)v.clone();
 }

 public void close()
 {
  frame.dispose();
  System.gc();
 }

 private class ButtonListener implements ActionListener
 {
  public void actionPerformed(ActionEvent e)
  {
   JButton jb = (JButton)e.getSource();
   if(jb == refreshButton)
   {
    getData(key); 
    if(error.length()==0)
    {
     td.initData(data);
     head.setModel(new ListMod(data.size()));
     table.revalidate();
     table.repaint();
     frame.invalidate();
     frame.validate();
     frame.repaint();
    }
   }
   else
    close();
   if(error.length()>0)
   {
    Mandalatama.errorDisplay(error.toString(),"ERROR");
    error.delete(0,error.length());
    close();
   }
  }
 }


 private class tableData extends AbstractTableModel
 {
  public final int TID = 0; // Trans Invoice Desc 
  public final int TIV = 1; // Trans Invoice Value 

  public Vector dataRow = null;
  public Vector colNames= null;
  
  public tableData(Vector ve)
  {
   initColumn();
   initData(ve);
  }
  public void initData(Vector ve)
  { dataRow = ve; }
  public void initColumn()
  {
   if(colNames == null) colNames = new Vector();
   colNames.add(new String("Description"));
   colNames.add(new String("Value"));
  }
  public int getRowCount()
  { return dataRow.size(); }
  public int getColumnCount()
  { return colNames.size(); }
  public String getColumnName(int column)
  { return (String)colNames.elementAt(column); }
  public Class getColumnClass(int column)
  { return getValueAt(0,column).getClass(); }
  public boolean isCellEditable(int row,int column)
  { return false; }
  public int getColumnType(int column)
  {
   return TableSorter.STRING; // STRING
  }
  public Object getValueAt(int row,int column)
  {
   if(row<0 || row>=getRowCount())
    return " ";
   transData trans = (transData)dataRow.elementAt(row);
   switch(column)
   {
    case TID : return trans.getDesc();
    case TIV : return trans.getValue();
   }
   return " ";
  }
  public void setValueAt(Object val,int row,int column)
  {
   if(row<0 || row>=getRowCount()) return;
   transData trans = (transData)dataRow.elementAt(row);
   switch(column)
   {
    case TID : trans.setDesc((String)val); break;
    case TIV : trans.setValue((String)val); break;
   }
   dataRow.setElementAt(trans,row);
   fireTableCellUpdated(row,column);
  }
 }

 public class transData
 {
  private String transDesc;
  private String transValue;
  public transData() { }

  public transData(String d,String v)
  {
   setDesc(d);
   setValue(v);
  }
  public void setDesc(String d)
  { transDesc= new String(d); }
  public void setValue(String v)
  { transValue = new String(v); }

  public String getDesc()
  { return transDesc; }
  public String getValue()
  { return transValue; }
 }
}
