import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.sql.SQLException;
import java.util.Vector;

public class ViewUser
{
 private JTable table;
 private JFrame frame;
 private tableData td;
 private StringBuffer error = new StringBuffer(128);
 private Vector masterData = null;
 private JButton closeB = new JButton("Close");
 private JButton refreshB = new JButton("Refresh");
 
 public ViewUser()
 {
  masterData = fetchData();
  if(masterData == null)
  {
   Mandalatama.infoDisplay("Currently there is no user in database","Info");
   return;
  }
  if(error.length()>0)
  {
   Mandalatama.errorDisplay(error.toString(),"ERROR");
   return;
  }
  frame = new JFrame("View User");
  frame.addWindowListener(new WindowAdapter()
  {
   public void windowClosing(WindowEvent e)
   {
    Mandalatama.infoDisplay("Click on Close in the panel instead","Information");
    frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    return;
   }
  });
  frame.getContentPane().setLayout(new BorderLayout());
  frame.getContentPane().add(createMain(),BorderLayout.NORTH);
  frame.getContentPane().add(createBottom(),BorderLayout.SOUTH);
  frame.pack();
  frame.setVisible(true);
 }

 private Vector fetchData()
 {
  DBConnect db = new DBConnect();
  Vector v = null;
  try
  {
   if(!db.makeConnection("MANDALATAMA"))
    throw new SQLException("Failure to make connection\n");
   v = db.QueryMulti("authorizedUser","*",
   "ORDER BY authorizedLevel AND authorizedValid;");
   if(v!=null)
    v = refineResult(v);
  }
  catch(SQLException sqle)
  {
   sqle.printStackTrace(System.out);
   error.append(sqle.getMessage());
   if(v!=null&&v.size()>0) v.removeAllElements();
   v = null;
  }
  finally
  {
   db.close();
  }
  return v;
 }

 private Vector refineResult(Vector v)
 {
  Vector v2 = new Vector();
  Encap e = null;
  do
  {
   e = (Encap)v.remove(0);
   UserInfo ui = new UserInfo();
   ui.setUserLevel(e.extractInteger("authorizedLevel",0));
   ui.setUserName(e.extractString("authorizedName",null));
   if(e.extractString("authorizedValid",null).equalsIgnoreCase("Y"))
    ui.setUserValid("YES");
   else 
    ui.setUserValid("NO");
   v2.add(ui);
  }while(v.size()>0);
  return v2;
 }

 private JPanel createBottom()
 {
  JPanel panel = new JPanel(new FlowLayout());
  ButtonListener bl = new ButtonListener();
  closeB.addActionListener(bl);
  refreshB.addActionListener(bl);
  panel.add(refreshB);
  panel.add(closeB);
  return panel;
 }

 private JPanel createMain()
 {
  JPanel panel = new JPanel();
  td = new tableData(masterData);
  table = new JTable();
  table.setAutoCreateColumnsFromModel(false);
  table.setRowHeight(20);
  table.setModel(td);
  table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
  MultiLineHeaderRender mlhr = new MultiLineHeaderRender();
  int len = td.getColumnCount();
  for(int i=0;i<len;i++)
  {
   TableCellRenderer render = new DefaultTableCellRenderer();
   TableCellEditor editor = new DefaultCellEditor(new JTextField());
   TableColumn column;
   column = new TableColumn(i,30,render,editor);
   column.setHeaderRenderer(mlhr);
   table.addColumn(column);
  }
  JList head = new JList(new ListMod(masterData.size()));
  head.setFixedCellWidth(20);
  head.setFixedCellHeight(table.getRowHeight());
  head.setCellRenderer(new RowHeaderRenderer(table));
  panel.setPreferredSize(new Dimension(600,400));
  panel.setLayout(new BorderLayout());
  JScrollPane jsp = new JScrollPane(table);
  jsp.setRowHeaderView(head);
  panel.add(jsp,BorderLayout.CENTER);	
  return panel;
 }

 public void close()
 {
  frame.dispose();
  System.gc();
 }

 private class ButtonListener implements ActionListener
 {
  public void actionPerformed(ActionEvent e)
  {
   JButton jb = (JButton)e.getSource();
   if(jb == refreshB)
   {
    if(masterData!=null&&masterData.size()>0)
     masterData.removeAllElements();
    masterData = fetchData();
    if(error.length()==0&&masterData!=null)
     displayTable(masterData);
    else if(masterData==null)
     error.append("No more data available in database\n");
   }
   else close();
   if(error.length()>0)
   {
    Mandalatama.errorDisplay(error.toString(),"ERROR");
    error.delete(0,error.length());
   }
  }
  public void displayTable(Vector v)
  {
   td.initData(v);
   table.revalidate();
   table.repaint();
   frame.invalidate();
   frame.validate();
   frame.repaint();
  }
 }

 private class tableData extends AbstractTableModel
 {
  public final int USRLEVEL = 0;
  public final int USRNAME = 1;
  public final int USRACTIVE = 2;

  public Vector dataRow = null;
  public final String[] colNames = {"User Level","User Name","User Valid"};
	
  public tableData(Vector v)
  { initData(v); }
  public void initData(Vector v)
  { dataRow = v; }
  public int getRowCount()
  { return dataRow.size(); }
  public int getColumnCount()
  { return colNames.length; }
  public String getColumnName(int column)
  { return colNames[column]; }
  public Class getColumnClass(int column)
  { return getValueAt(0,column).getClass(); }
  public boolean isCellEditable(int row,int column)
  { return false; }
  public Object getValueAt(int row,int column)
  {
   if(row<0||row>=getRowCount()) return " ";
   UserInfo user = (UserInfo)dataRow.get(row);
   switch(column)
   {
    case USRLEVEL : return user.getUserLevel();
    case USRNAME  : return user.getUserName();
    case USRACTIVE: return user.getUserValid();
   }
   return " ";
  }
  public void setValueAt(Object obj,int row,int column)
  {
   if(row<0||row>=getRowCount()) return;
   UserInfo user = (UserInfo)dataRow.get(row);
   switch(column)
   {
    case USRLEVEL  : user.setUserLevel(((Integer)obj).intValue()); break;
    case USRNAME   : user.setUserName((String)obj); break;
    case USRACTIVE : user.setUserValid((String)obj); break;
   }
   dataRow.setElementAt(user,row);
   fireTableCellUpdated(row,column);
  }
 }

 private class UserInfo
 {
  private int userLevel = 0;
  private StringBuffer userName = new StringBuffer(32);
  private StringBuffer userValid = new StringBuffer(3);
  public UserInfo() { }
  public UserInfo(int lvl,String name,String valid)
  {
   setUserLevel(lvl);
   setUserName(name);
   setUserValid(valid);
  }
  public void setUserLevel(int lvl)
  { userLevel = lvl; }
  public void setUserName(String name)
  {
   if(userName.length()>0) userName.delete(0,userName.length());
   userName.append(name);
  }
  public void setUserValid(String valid)
  {
   if(userValid.length()>0) userValid.delete(0,userValid.length());
   userValid.append(valid);
  }
  public Integer getUserLevel()
  { return new Integer(userLevel); }
  public String getUserName()
  { return userName.toString(); }
  public String getUserValid()
  { return userValid.toString(); }
 }
}
